@@ -40,22 +40,22 @@
 //! assert_eq!(scan_buffer[0], (7, 42));
 //! ```
 
-#![feature(core_intrinsics)]
-#![feature(generic_associated_types)]
 #![allow(clippy::comparison_chain)]
 #![allow(clippy::len_without_is_empty)]
 
 mod base_node;
-mod child_ptr;
 mod key;
 mod lock;
 mod node_16;
 mod node_256;
 mod node_4;
 mod node_48;
+mod node_ptr;
 mod tree;
 mod utils;
 
+mod node_lock;
+
 mod range_scan;
 
 #[cfg(feature = "stats")]
@@ -64,9 +64,12 @@ mod stats;
 #[cfg(test)]
 mod tests;
 
+use std::vec;
+
+use base_node::BaseNode;
 pub use key::RawKey;
 use key::UsizeKey;
-pub use tree::RawTree as RawArt;
+pub use tree::RawTree;
 
 /// Types needed to safely access shared data concurrently.
 pub mod epoch {
@@ -76,7 +79,7 @@ pub mod epoch {
 /// ArtUsize is a special case for [Art] where the key is a usize.
 /// It can have better performance
 pub struct ArtUsize {
-    inner: RawArt<UsizeKey>,
+    inner: RawTree<UsizeKey>,
 }
 
 impl Default for ArtUsize {
@@ -130,7 +133,7 @@ impl ArtUsize {
     #[inline]
     pub fn new() -> Self {
         ArtUsize {
-            inner: RawArt::new(),
+            inner: RawTree::new(),
         }
     }
 
@@ -154,7 +157,7 @@ impl ArtUsize {
         self.inner.remove(&key, guard)
     }
 
-    /// Insert a key-value pair to the tree.
+    /// Insert a key-value pair to the tree, returns the previous value if the key was already present.
     ///
     /// # Examples
     ///
@@ -165,9 +168,11 @@ impl ArtUsize {
     ///
     /// tree.insert(1, 42, &guard);
     /// assert_eq!(tree.get(&1, &guard).unwrap(), 42);
+    /// let old = tree.insert(1, 43, &guard);
+    /// assert_eq!(old, Some(42));
     /// ```
     #[inline]
-    pub fn insert(&self, k: usize, v: usize, guard: &epoch::Guard) {
+    pub fn insert(&self, k: usize, v: usize, guard: &epoch::Guard) -> Option<usize> {
         let key = UsizeKey::key_from(k);
         self.inner.insert(key, v, guard)
     }
@@ -215,7 +220,7 @@ impl ArtUsize {
 
 /// The main adaptive radix tree.
 pub struct Art<V: Clone> {
-    inner: RawArt<UsizeKey>,
+    inner: RawTree<UsizeKey>,
     pt: std::marker::PhantomData<V>,
 }
 
@@ -225,6 +230,33 @@ impl<V: Clone> Default for Art<V> {
     }
 }
 
+impl<V: Clone> Drop for Art<V> {
+    fn drop(&mut self) {
+        let v = unsafe { std::mem::ManuallyDrop::take(&mut self.inner.root) };
+
+        let mut sub_nodes = vec![(Box::into_raw(v) as *const BaseNode, 0)];
+
+        while !sub_nodes.is_empty() {
+            let (node, level) = sub_nodes.pop().unwrap();
+            let children = unsafe { &*node }.get_children(0, 255);
+            for (_k, n) in children {
+                if level != 7 {
+                    sub_nodes.push((
+                        n.as_ptr(),
+                        level + 1 + unsafe { &*n.as_ptr() }.prefix().len(),
+                    ));
+                } else {
+                    let payload = n.as_tid() as *mut V;
+                    unsafe { std::mem::drop(Box::from_raw(payload)) };
+                }
+            }
+            unsafe {
+                std::ptr::drop_in_place(node as *mut BaseNode);
+            }
+        }
+    }
+}
+
 impl<V: Clone> Art<V> {
     /// Create an empty [Art] tree.
     ///
@@ -236,7 +268,7 @@ impl<V: Clone> Art<V> {
     /// ```
     pub fn new() -> Self {
         Art {
-            inner: RawArt::new(),
+            inner: RawTree::new(),
             pt: std::marker::PhantomData,
         }
     }
@@ -256,7 +288,7 @@ impl<V: Clone> Art<V> {
         crossbeam_epoch::pin()
     }
 
-    /// Insert a key-value pair to the tree.
+    /// Insert a key-value pair to the tree, return the previous value if the key was found.
     ///
     /// # Examples
     ///
@@ -267,12 +299,17 @@ impl<V: Clone> Art<V> {
     ///
     /// tree.insert(1, "42".to_string(), &guard);
     /// assert_eq!(tree.get(&1, &guard).unwrap(), "42".to_string());
+    /// let old = tree.insert(1, "43".to_string(), &guard);
+    /// assert_eq!(old, Some("42".to_string()));
     /// ```
-    pub fn insert(&self, k: usize, v: V, guard: &epoch::Guard) {
+    pub fn insert(&self, k: usize, v: V, guard: &epoch::Guard) -> Option<V> {
         let key = UsizeKey::key_from(k);
         let boxed = Box::new(v);
         let boxed_ptr = Box::into_raw(boxed) as *mut V;
-        self.inner.insert(key, boxed_ptr as usize, guard)
+        let old = self.inner.insert(key, boxed_ptr as usize, guard)?;
+        let val = unsafe { Box::from_raw(old as *mut V) };
+        let val = *val;
+        Some(val)
     }
 
     /// Returns a copy of the value corresponding to the key.
