@@ -1,18 +1,32 @@
 use crate::{
     base_node::{BaseNode, Node, NodeIter, NodeType},
-    child_ptr::NodePtr,
+    node_ptr::NodePtr,
 };
 
 pub(crate) const EMPTY_MARKER: u8 = 48;
 
 #[repr(C)]
+#[repr(align(64))]
 pub(crate) struct Node48 {
     base: BaseNode,
 
+    next_empty: u8,
     pub(crate) child_idx: [u8; 256],
     children: [NodePtr; 48],
 }
 
+impl Node48 {
+    pub(crate) fn init_empty(&mut self) {
+        for v in self.child_idx.iter_mut() {
+            *v = EMPTY_MARKER;
+        }
+        self.next_empty = 0;
+        for (i, child) in self.children.iter_mut().enumerate() {
+            *child = NodePtr::from_tid(i + 1);
+        }
+    }
+}
+
 pub(crate) struct Node48Iter<'a> {
     start: u16,
     end: u16,
@@ -46,9 +60,11 @@ impl Node for Node48 {
 
     fn remove(&mut self, k: u8) {
         debug_assert!(self.child_idx[k as usize] != EMPTY_MARKER);
-        self.children[self.child_idx[k as usize] as usize] = NodePtr::from_null();
+        let pos = self.child_idx[k as usize];
+        self.children[pos as usize] = NodePtr::from_tid(self.next_empty as usize);
         self.child_idx[k as usize] = EMPTY_MARKER;
-        self.base.count -= 1;
+        self.next_empty = pos;
+        self.base.meta.count -= 1;
         debug_assert!(self.get_child(k).is_none());
     }
 
@@ -77,31 +93,28 @@ impl Node for Node48 {
     }
 
     fn is_full(&self) -> bool {
-        self.base.count == 48
+        self.base.meta.count == 48
     }
 
     fn is_under_full(&self) -> bool {
-        self.base.count == 12
+        self.base.meta.count == 12
     }
 
     fn insert(&mut self, key: u8, node: NodePtr) {
-        let mut pos = self.base.count as usize;
+        let pos = self.next_empty as usize;
+        self.next_empty = self.children[pos as usize].as_tid() as u8;
 
-        if !self.children[pos].is_null() {
-            pos = 0;
-            while !self.children[pos].is_null() {
-                pos += 1;
-            }
-        }
         debug_assert!(pos < 48);
 
         self.children[pos] = node;
         self.child_idx[key as usize] = pos as u8;
-        self.base.count += 1;
+        self.base.meta.count += 1;
     }
 
-    fn change(&mut self, key: u8, val: NodePtr) {
+    fn change(&mut self, key: u8, val: NodePtr) -> NodePtr {
+        let old = self.children[self.child_idx[key as usize] as usize];
         self.children[self.child_idx[key as usize] as usize] = val;
+        old
     }
 
     fn get_child(&self, key: u8) -> Option<NodePtr> {
